@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2010-2023 Vaclav Slavik
+ *  Copyright (C) 2010-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -27,9 +27,9 @@
 #define _CAT_SORTING_H_
 
 #include "catalog.h"
+#include "unicode_helpers.h"
 
 #include <memory>
-#include <unicode/coll.h>
 
 /// Sort order information
 struct SortOrder
@@ -62,6 +62,7 @@ struct SortOrder
     bool errorsFirst;
 };
 
+
 /**
     Comparator for sorting catalog items by different criteria.
  */
@@ -80,12 +81,36 @@ public:
 
 protected:
     const CatalogItem& Item(int i) const { return *m_catalog[i]; }
-    int CompareStrings(wxString a, wxString b) const;
+
+    // Pre-process given string and return it in a form efficient for comparing
+    // with ICU collator. This does two things:
+    //  1. Converts to UTF-16 (matters on non-Windows platforms where wchar_t is UTF-32)
+    //  2. Perform substitution of accelerator characters in the string
+    static str::UCharBuffer ConvertToSortKey(const wxString& a)
+    {
+        if (a.find_first_of(L"&_") == wxString::npos)
+        {
+            return str::to_icu(a);
+        }
+        else
+        {
+            wxString a_(a);
+            a_.Replace("&", "");
+            a_.Replace("_", "");
+
+            auto buf = str::to_icu(a_);
+            // on Windows to_icu() returns shallow view of the string, we need to make
+            // a deep copy because a_ is a local temporary variable:
+            buf.ensure_owned();
+            return buf;
+        }
+    }
 
 private:
     const Catalog& m_catalog;
     SortOrder m_order;
-    std::unique_ptr<icu::Collator> m_collator;
+    std::unique_ptr<unicode::Collator> m_collator;
+    std::vector<str::UCharBuffer> m_sortKeys;
 };
 
 
