@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2023 Vaclav Slavik
+ *  Copyright (C) 1999-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -47,24 +47,6 @@ typedef std::shared_ptr<CatalogItem> CatalogItemPtr;
 typedef std::shared_ptr<Catalog> CatalogPtr;
 
 
-/// The possible bookmarks for a given item
-typedef enum
-{
-    NO_BOOKMARK = -1,
-    BOOKMARK_0,
-    BOOKMARK_1,
-    BOOKMARK_2,
-    BOOKMARK_3,
-    BOOKMARK_4,
-    BOOKMARK_5,
-    BOOKMARK_6,
-    BOOKMARK_7,
-    BOOKMARK_8,
-    BOOKMARK_9,
-    BOOKMARK_LAST
-} Bookmark;
-
-
 /**
     Optional data attached to CatalogItem.
 
@@ -112,8 +94,8 @@ class CatalogItem
                   m_isTranslated(false),
                   m_isModified(false),
                   m_isPreTranslated(false),
-                  m_lineNum(0),
-                  m_bookmark(NO_BOOKMARK) {}
+                  m_lineNum(0)
+        {}
 
         CatalogItem(const CatalogItem&) = delete;
 
@@ -188,7 +170,7 @@ class CatalogItem
 
         /// Returns format flag ("php" for "php-format" etc.) if there's any,
         // empty string otherwise
-        wxString GetFormatFlag() const;
+        std::string GetFormatFlag() const;
 
         /// Like GetFormatFlags(), but only for internal uses (e.g. fileformat-specific highlighting)
         virtual std::string GetInternalFormatFlag() const { return std::string(); }
@@ -208,11 +190,6 @@ class CatalogItem
         wxString GetOldMsgid() const;
         bool HasOldMsgid() const { return !m_oldMsgid.empty(); }
 
-        /// Returns the bookmark for the item
-        Bookmark GetBookmark() const {return m_bookmark;}
-        /// Returns true if the item has a bookmark
-        bool HasBookmark() const {return (GetBookmark() != NO_BOOKMARK);}
-
 
         // -------------------------------------------------------------------
         // Setters for user-editable values:
@@ -240,8 +217,6 @@ class CatalogItem
         void SetModified(bool modified) { m_isModified = modified; }
         /// Sets pre-translated translation flag.
         void SetPreTranslated(bool pre) { m_isPreTranslated = pre; }
-        /// Sets the bookmark
-        void SetBookmark(Bookmark bookmark) {m_bookmark = bookmark;}
 
         /// Sets the comment.
         void SetComment(const wxString& c) { m_comment = c; }
@@ -340,7 +315,6 @@ class CatalogItem
         wxString m_moreFlags;
         wxString m_comment;
         int m_lineNum;
-        Bookmark m_bookmark;
 
         std::shared_ptr<Issue> m_issue;
         std::shared_ptr<SideloadedItemData> m_sideloaded;
@@ -431,7 +405,6 @@ class Catalog
             Language Lang;
 
             wxArrayString SearchPaths, SearchPathsExcluded, Keywords;
-            int Bookmarks[BOOKMARK_LAST];
             wxString BasePath;
 
             wxString Comment;
@@ -601,22 +574,15 @@ class Catalog
         /// Removes all obsolete translations from the catalog
         virtual void RemoveDeletedItems() = 0;
 
+        /// Removes translations identical to the source text, returns true if any changes were made
+        bool RemoveSameAsSourceTranslations();
+
         /// Finds item by line number
         CatalogItemPtr FindItemByLine(int lineno);
 
         /// Finds catalog index by line number
         int FindItemIndexByLine(int lineno);
 
-        /// Sets the given item to have the given bookmark and returns the index
-        /// of the item that previously had this bookmark (or -1)
-        int SetBookmark(int id, Bookmark bookmark);
-
-        /// Returns the index of the item that has the given bookmark or -1
-        int GetBookmarkIndex(Bookmark bookmark) const
-        {
-            return m_header.Bookmarks[bookmark];
-        }
-
 
         /// Validates correctness of the translation by running msgfmt
         /// Returns number of errors (i.e. 0 if no errors).
