@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2023 Vaclav Slavik
+ *  Copyright (C) 2023-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -33,6 +33,7 @@
 #include "http_client.h"
 #include "languagectrl.h"
 #include "str_helpers.h"
+#include "unicode_helpers.h"
 #include "utility.h"
 
 #include "cloud_sync.h"
@@ -41,9 +42,6 @@
 #include "localazy_client.h"
 #include "localazy_gui.h"
 
-#include <unicode/coll.h>
-#include <boost/algorithm/string.hpp>
-
 #include <wx/choice.h>
 #include <wx/renderer.h>
 #include <wx/simplebook.h>
@@ -54,10 +52,6 @@
     #include <wx/generic/private/markuptext.h>
 #endif
 
-#if !wxCHECK_VERSION(3,1,0)
-    #define CenterVertical() Center()
-#endif
-
 namespace
 {
 
@@ -98,7 +92,7 @@ void ServiceSelectionPanel::AddService(AccountDetailPanel *account)
 
     m_sizer->Insert(pos, content, wxSizerFlags(1).Expand());
     if (!isFirst)
-        m_sizer->Insert(posLine, new StaticLine(this, wxID_ANY), wxSizerFlags().Expand().Border(wxTOP|wxBOTTOM, PX(24)));
+        m_sizer->Insert(posLine, new wxStaticLine(this, wxID_ANY), wxSizerFlags().Expand().Border(wxTOP|wxBOTTOM, PX(24)));
 }
 
 
@@ -126,7 +120,7 @@ wxSizer *ServiceSelectionPanel::CreateServiceContent(AccountDetailPanel *account
 
     auto buttons = new wxBoxSizer(wxHORIZONTAL);
     sizer->Add(buttons, wxSizerFlags().Expand().Border(wxTOP, PX(16)));
-    buttons->Add(learnMore, wxSizerFlags().Center().Border(wxLEFT, PX(LearnMoreLink::EXTRA_INDENT)));
+    buttons->Add(learnMore, wxSizerFlags().Center());
     buttons->AddStretchSpacer();
     buttons->Add(signIn, wxSizerFlags());
 
@@ -143,7 +137,7 @@ AccountsPanel::AccountsPanel(wxWindow *parent, int flags) : wxPanel(parent, wxID
                   wxSizerFlags().Expand().Border(wxBOTTOM, PX(2)));
 
     auto learn = new LearnMoreLink(this, "https://poedit.net/cloud-sync", _("How does cloud sync work?"));
-    topsizer->Add(learn, wxSizerFlags().Left().Border(wxLEFT, PX(LearnMoreLink::EXTRA_INDENT)));
+    topsizer->Add(learn, wxSizerFlags().Left());
     topsizer->AddSpacer(2 * PXDefaultBorder);
 
     wxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
@@ -289,28 +283,11 @@ namespace
 template<typename T, typename Key>
 void SortAlphabetically(std::vector<T>& items, Key func)
 {
-    UErrorCode err = U_ZERO_ERROR;
-    std::unique_ptr<icu::Collator> coll(icu::Collator::createInstance(err));
-    if (coll)
-        coll->setStrength(icu::Collator::SECONDARY); // case insensitive
-
+    unicode::Collator coll(unicode::Collator::case_insensitive);
     std::sort
     (
         items.begin(), items.end(),
-        [&coll,&func](const T& a, const T& b)
-        {
-            auto ka = func(a);
-            auto kb = func(b);
-            if (coll)
-            {
-                UErrorCode e = U_ZERO_ERROR;
-                return coll->compare(str::to_icu(ka), str::to_icu(kb), e) == UCOL_LESS;
-            }
-            else
-            {
-                return ka < kb;
-            }
-        }
+        [&coll,&func](const T& a, const T& b){ return coll(func(a), func(b)); }
     );
 }
 
@@ -330,9 +307,7 @@ public:
         : wxDataViewListCtrl(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                              wxDV_NO_HEADER | MSW_OR_OTHER(wxBORDER_SIMPLE, wxBORDER_SUNKEN))
     {
-    #if wxCHECK_VERSION(3,1,1)
         SetRowHeight(PX(36));
-    #endif
         SetMinSize(wxSize(PX(500), PX(200)));
     #ifdef __WXOSX__
         if (@available(macOS 11.0, *))
@@ -375,7 +350,6 @@ private:
 
         for (auto& f : m_files)
         {
-        #if wxCHECK_VERSION(3,1,1)
             wxString text = wxString::Format
             (
                 "%s\n<small><span %s>%s</span></small>",
@@ -383,10 +357,6 @@ private:
                 secondaryFormatting,
                 EscapeMarkup(f.description)
             );
-        #else
-            wxString text(f.title);
-        #endif
-
             wxVector<wxVariant> data;
             data.push_back({text});
             AppendItem(data);
@@ -398,9 +368,7 @@ private:
     public:
         MultilineTextRenderer() : wxDataViewTextRenderer()
         {
-    #if wxCHECK_VERSION(3,1,1)
             EnableMarkup();
-    #endif
         }
 
     #ifdef __WXMSW__
@@ -410,11 +378,12 @@ private:
             if ( state & wxDATAVIEW_CELL_SELECTED )
                 flags |= wxCONTROL_SELECTED;
 
+            rect.height /= 2;
             for (auto& line: wxSplit(m_text, '\n'))
             {
                 wxItemMarkupText markup(line);
                 markup.Render(GetView(), *dc, rect, flags, GetEllipsizeMode());
-                rect.y += rect.height / 2;
+                rect.y += rect.height;
             }
 
             return true;
@@ -453,7 +422,7 @@ public:
         loginSizer->Add(m_loginImage, wxSizerFlags().ReserveSpaceEvenIfHidden().Center());
         loginSizer->Add(m_loginText, wxSizerFlags().ReserveSpaceEvenIfHidden().Center().Border(wxLEFT, PX(2)));
         auto manageLink = new LearnMoreLink(this, "", _("Manage accounts"));
-        loginSizer->Add(manageLink, wxSizerFlags().Center().Border(wxRIGHT, PX(LearnMoreLink::EXTRA_INDENT)));
+        loginSizer->Add(manageLink, wxSizerFlags().Center());
         m_loginText->Hide();
         m_loginImage->Hide();
 
@@ -603,6 +572,7 @@ private:
     {
         SortAlphabetically(m_projects, [](const auto& p){ return p.name; });
 
+        m_project->Clear();
         m_project->Append("");
         for (auto& p: m_projects)
             m_project->Append(p.name);
@@ -732,9 +702,8 @@ private:
 
     void OnManageAccounts(wxHyperlinkEvent&)
     {
-        ManageAccounts<CloudEditLoginDialog<AccountsPanel>>([=](bool ok){
-            if (ok)
-                LoadFromCloud(nullptr);
+        ManageAccounts<CloudEditLoginDialog<AccountsPanel>>([=](bool /*ok*/) {
+            LoadFromCloud(nullptr);
         });
     }
 
@@ -859,12 +828,12 @@ void CloudOpenFile(wxWindow *parent,
 bool ShouldSyncToCloudAutomatically(CatalogPtr catalog)
 {
     auto root = wxFileName::DirName(GetCacheDir());
-    root.Normalize();
+    root.MakeAbsolute();
 
     wxFileName f(catalog->GetFileName());
-    f.Normalize();
+    f.MakeAbsolute();
 
-    return f.GetFullPath().StartsWith(root.GetFullPath());
+    return f.GetFullPath().starts_with(root.GetFullPath());
 }
 
 
