@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2016-2023 Vaclav Slavik
+ *  Copyright (C) 2016-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -57,6 +57,9 @@ public:
     static bool UseTM() { return Read("/use_tm", true); }
     static void UseTM(bool use) { Write("/use_tm", use); }
 
+    static bool CheckForBetaUpdates() { return Read("/check_for_beta_updates", false); }
+    static void CheckForBetaUpdates(bool use) { Write("/check_for_beta_updates", use); }
+
     static ::PretranslateSettings PretranslateSettings();
     static void PretranslateSettings(::PretranslateSettings s);
 
