@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2016-2023 Vaclav Slavik
+ *  Copyright (C) 2016-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -473,7 +473,7 @@ void SwitchButton::OnMouseClick(wxMouseEvent& e)
 wxSize SwitchButton::DoGetBestSize() const
 {
     auto size = wxToggleButton::DoGetBestSize();
-    size.x += PX(38);
+    size.x += PX(44);
     size.y = PX(20);
     return size;
 }
@@ -675,16 +675,13 @@ TranslucentButton::TranslucentButton(wxWindow *parent, wxWindowID winid, const w
     wxButton::Create(parent, winid, label);
 
 #ifdef __WXMSW__
-    if (IsWindows10OrGreater())
+    ColorScheme::SetupWindowColors(this, [=]
     {
-        ColorScheme::SetupWindowColors(this, [=]
-        {
-            if (ColorScheme::GetAppMode() == ColorScheme::Light)
-                SetBackgroundColour(ColorScheme::GetBlendedOn(::Color::TranslucentButton, parent));
-            else
-                SetBackgroundColour(GetDefaultAttributes().colBg);
-        });
-    }
+        if (ColorScheme::GetAppMode() == ColorScheme::Light)
+            SetBackgroundColour(ColorScheme::GetBlendedOn(::Color::TranslucentButton, parent));
+        else
+            SetBackgroundColour(GetDefaultAttributes().colBg);
+    });
 #endif
 
 #ifdef __WXGTK3__
