@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2015-2023 Vaclav Slavik
+ *  Copyright (C) 2015-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,10 @@
 #include "errors.h"
 #include "hidpi.h"
 #include "utility.h"
+#include "str_helpers.h"
+#include "unicode_helpers.h"
 
+#include <wx/activityindicator.h>
 #include <wx/app.h>
 #include <wx/artprov.h>
 #include <wx/clipbrd.h>
@@ -47,20 +50,10 @@
 #include <wx/generic/private/markuptext.h>
 #endif
 
-#if wxCHECK_VERSION(3,1,0)
-    #include <wx/activityindicator.h>
-#else
-    #include "wx_backports/activityindicator.h"
-#endif
-
-#include <unicode/brkiter.h>
 #ifdef __WXGTK__
 #include <gtk/gtk.h>
 #endif
 
-#include "str_helpers.h"
-#include "unicode_helpers.h"
-
 #include <map>
 #include <memory>
 
@@ -80,16 +73,13 @@ wxString WrapTextAtWidth(const wxString& text_, int width, Language lang, wxWind
         
     auto text = str::to_icu(text_);
 
-    static std::map<std::string, std::shared_ptr<icu::BreakIterator>> lang_iters;
-    std::shared_ptr<icu::BreakIterator> iter;
+    static std::map<std::string, std::shared_ptr<unicode::BreakIterator>> lang_iters;
+    std::shared_ptr<unicode::BreakIterator> iter;
     auto lang_name = lang.IcuLocaleName();
     auto li = lang_iters.find(lang_name);
     if (li == lang_iters.end())
     {
-        UErrorCode err = U_ZERO_ERROR;
-        iter.reset(icu::BreakIterator::createLineInstance(lang.IsValid() ? lang.ToIcu() : icu::Locale(), err));
-        if (!iter)
-            iter.reset(icu::BreakIterator::createLineInstance(icu::Locale::getEnglish(), err));
+        iter.reset(new unicode::BreakIterator(UBRK_LINE, lang));
         lang_iters[lang_name] = iter;
     }
     else
@@ -97,7 +87,7 @@ wxString WrapTextAtWidth(const wxString& text_, int width, Language lang, wxWind
         iter = li->second;
     }
 
-    iter->setText(text);
+    iter->set_text(text);
 
     wxString out;
     out.reserve(text_.length() + 10);
@@ -105,14 +95,17 @@ wxString WrapTextAtWidth(const wxString& text_, int width, Language lang, wxWind
     int32_t lineStart = 0;
     wxString previousSubstr;
 
-    for (int32_t pos = iter->next(); pos != icu::BreakIterator::DONE; pos = iter->next())
+    for (int32_t pos = iter->begin(); pos != iter->end(); pos = iter->next())
     {
-        auto substr = str::to_wx(text.tempSubStringBetween(lineStart, pos));
+        if (pos == lineStart)
+            continue;
+        auto substr = str::to_wx(text + lineStart, pos - lineStart);
+        substr.Trim();
 
         if (wnd->GetTextExtent(substr).x > width)
         {
             auto previousPos = iter->previous();
-            if (previousPos == lineStart || previousPos == icu::BreakIterator::DONE)
+            if (previousPos == lineStart || previousPos == iter->end())
             {
                 // line is too large but we can't break it, so have no choice but not to wrap
                 out += substr;
@@ -150,8 +143,7 @@ wxString WrapTextAtWidth(const wxString& text_, int width, Language lang, wxWind
         out += previousSubstr;
     }
 
-    if (out.Last() == '\n')
-        out.RemoveLast();
+    out.Trim();
 
     return out;
 }
@@ -173,8 +165,8 @@ HeadingLabel::HeadingLabel(wxWindow *parent, const wxString& label)
 }
 
 
-AutoWrappingText::AutoWrappingText(wxWindow *parent, const wxString& label)
-    : wxStaticText(parent, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE),
+AutoWrappingText::AutoWrappingText(wxWindow *parent, wxWindowID winid, const wxString& label)
+    : wxStaticText(parent, winid, "", wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE),
       m_text(label),
       m_wrapWidth(-1)
 {
@@ -232,6 +224,18 @@ bool AutoWrappingText::InformFirstDirection(int direction, int size, int /*avail
     return false;
 }
 
+#ifdef __WXOSX__
+wxSize AutoWrappingText::DoGetBestSize() const
+{
+    auto sz = wxStaticText::DoGetBestSize();
+    // AppKit's intristicContentSize calculation is sometimes subtly wrong in our use case,
+    // hiding the last line of wrapped text. It seems to be off-by-two error only:
+    if (sz.y > 0)
+        sz.y += 2;
+    return sz;
+}
+#endif
+
 void AutoWrappingText::OnSize(wxSizeEvent& e)
 {
     e.Skip();
@@ -258,8 +262,8 @@ bool AutoWrappingText::RewrapForWidth(int width)
 }
 
 
-SelectableAutoWrappingText::SelectableAutoWrappingText(wxWindow *parent, const wxString& label)
-    : AutoWrappingText(parent, label)
+SelectableAutoWrappingText::SelectableAutoWrappingText(wxWindow *parent, wxWindowID winid, const wxString& label)
+    : AutoWrappingText(parent, winid, label)
 {
 #if defined(__WXOSX__)
     NSTextField *view = (NSTextField*)GetHandle();
@@ -269,7 +273,7 @@ SelectableAutoWrappingText::SelectableAutoWrappingText(wxWindow *parent, const w
     gtk_label_set_selectable(view, TRUE);
 #else
     // at least allow copying
-    static wxWindowID idCopy = wxNewId();
+    static wxWindowIDRef idCopy = NewControlId();
     Bind(wxEVT_CONTEXT_MENU, [=](wxContextMenuEvent&){
         wxMenu menu;
         menu.Append(idCopy, _("&Copy"));
@@ -284,16 +288,10 @@ SelectableAutoWrappingText::SelectableAutoWrappingText(wxWindow *parent, const w
 
 
 ExplanationLabel::ExplanationLabel(wxWindow *parent, const wxString& label)
-    : AutoWrappingText(parent, label)
+    : AutoWrappingText(parent, wxID_ANY, label)
 {
-#if defined(__WXOSX__)
+#if defined(__WXOSX__) || defined(__WXGTK__)
     SetWindowVariant(wxWINDOW_VARIANT_SMALL);
-#elif defined(__WXGTK__)
-    #if wxCHECK_VERSION(3,1,0)
-        SetWindowVariant(wxWINDOW_VARIANT_SMALL);
-    #else
-        SetFont(GetFont().Smaller());
-    #endif
 #endif
 #ifndef __WXGTK__
     ColorScheme::SetupWindowColors(this, [=]
@@ -307,14 +305,8 @@ ExplanationLabel::ExplanationLabel(wxWindow *parent, const wxString& label)
 SecondaryLabel::SecondaryLabel(wxWindow *parent, const wxString& label)
     : wxStaticText(parent, wxID_ANY, label)
 {
-#if defined(__WXOSX__)
+#if defined(__WXOSX__) || defined(__WXGTK__)
     SetWindowVariant(wxWINDOW_VARIANT_SMALL);
-#elif defined(__WXGTK__)
-    #if wxCHECK_VERSION(3,1,0)
-        SetWindowVariant(wxWINDOW_VARIANT_SMALL);
-    #else
-        SetFont(GetFont().Smaller());
-    #endif
 #endif
 #ifndef __WXGTK__
     ColorScheme::SetupWindowColors(this, [=]
@@ -474,7 +466,7 @@ ImageButton::ImageButton(wxWindow *parent, const wxString& bitmapName)
     view.buttonType = NSButtonTypeMomentaryChange;
 #else
     // refresh template icons on theme change (macOS handles automatically):
-    if (bitmapName.EndsWith("Template"))
+    if (bitmapName.ends_with("Template"))
     {
         ColorScheme::SetupWindowColors(this, [=]
         {
@@ -492,7 +484,7 @@ StaticBitmap::StaticBitmap(wxWindow *parent, const wxString& bitmapName)
 {
 #ifndef __WXOSX__
     // refresh template icons on theme change (macOS handles automatically):
-    if (bitmapName.EndsWith("Template"))
+    if (bitmapName.ends_with("Template"))
     {
         ColorScheme::SetupWindowColors(this, [=]
         {
@@ -612,9 +604,7 @@ class IconAndSubtitleListCtrl::MultilineTextRenderer : public wxDataViewTextRend
 public:
     MultilineTextRenderer() : wxDataViewTextRenderer()
     {
-#if wxCHECK_VERSION(3,1,1)
         EnableMarkup();
-#endif
     }
 
 #ifdef __WXMSW__
@@ -624,11 +614,12 @@ public:
         if ( state & wxDATAVIEW_CELL_SELECTED )
             flags |= wxCONTROL_SELECTED;
 
+        rect.height /= 2;
         for (auto& line: wxSplit(m_text, '\n'))
         {
             wxItemMarkupText markup(line);
             markup.Render(GetView(), *dc, rect, flags, GetEllipsizeMode());
-            rect.y += rect.height / 2;
+            rect.y += rect.height;
         }
 
         return true;
@@ -663,9 +654,7 @@ IconAndSubtitleListCtrl::IconAndSubtitleListCtrl(wxWindow *parent, const wxStrin
     const int icon_column_width = wxSystemSettings::GetMetric(wxSYS_ICON_X) + PX(12);
 #endif
 
-#if wxCHECK_VERSION(3,1,1)
     SetRowHeight(GetDefaultRowHeight());
-#endif
 
     AppendBitmapColumn("", 0, wxDATAVIEW_CELL_INERT, icon_column_width);
     auto renderer = new MultilineTextRenderer();
@@ -690,7 +679,6 @@ wxString IconAndSubtitleListCtrl::FormatItemText(const wxString& title, const wx
     auto secondaryFormatting = GetSecondaryFormatting();
 #endif
 
-#if wxCHECK_VERSION(3,1,1)
     return wxString::Format
     (
         "%s\n<small><span %s>%s</span></small>",
@@ -698,9 +686,6 @@ wxString IconAndSubtitleListCtrl::FormatItemText(const wxString& title, const wx
         secondaryFormatting,
         EscapeMarkup(description)
     );
-#else
-    return title;
-#endif
 }
 
 #ifndef __WXGTK__
@@ -740,12 +725,3 @@ void IconAndSubtitleListCtrl::UpdateFormattedItem(unsigned row, const wxString&
 {
     SetTextValue(FormatItemText(title, description), row, 1);
 }
-
-
-#if defined(__WXOSX__) && !wxCHECK_VERSION(3,2,3)
-StaticLine::StaticLine(wxWindow *parent, wxWindowID id) : wxStaticLine(parent, id)
-{
-    NSBox *box = (NSBox*)GetHandle();
-    box.boxType = NSBoxSeparator;
-}
-#endif
