@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2023 Vaclav Slavik
+ *  Copyright (C) 1999-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -55,13 +55,12 @@
 #include <boost/iostreams/filter/gzip.hpp>
 
 #ifdef __WXOSX__
-#include "macos_helpers.h"
+#import "PFMoveApplication.h"
 #endif
 
 #ifdef __WXMSW__
 #include <wx/msw/wrapwin.h>
 #include <Shlwapi.h>
-#include <winsparkle.h>
 #endif
 
 #ifdef __WXGTK__
@@ -70,12 +69,13 @@
 #endif
 
 #include <unicode/uclean.h>
-#include <unicode/putil.h>
+#include <unicode/uversion.h>
 
 #if !wxUSE_UNICODE
     #error "Unicode build of wxWidgets is required by Poedit"
 #endif
 
+#include "app_updates.h"
 #include "colorscheme.h"
 #include "concurrency.h"
 #include "configuration.h"
@@ -104,15 +104,6 @@
 #include "language.h"
 #include "welcomescreen.h"
 
-#ifdef __WXOSX__
-struct PoeditApp::NativeMacAppData
-{
-#ifdef USE_SPARKLE
-    NSObject *sparkleDelegate = nullptr;
-#endif
-};
-#endif
-
 
 #ifndef __WXOSX__
 
@@ -280,9 +271,6 @@ IMPLEMENT_APP(PoeditApp);
 
 PoeditApp::PoeditApp()
 {
-#ifdef __WXOSX__
-    m_nativeMacAppData.reset(new NativeMacAppData);
-#endif
 }
 
 PoeditApp::~PoeditApp()
@@ -326,11 +314,6 @@ wxString PoeditApp::GetAppBuildNumber() const
 #endif
 }
 
-bool PoeditApp::CheckForBetaUpdates() const
-{
-    return wxConfigBase::Get()->ReadBool("check_for_beta_updates", false);
-}
-
 
 #ifndef __WXOSX__
 static wxArrayString gs_filesToOpen;
@@ -386,7 +369,7 @@ bool PoeditApp::OnInit()
     InitHiDPIHandling();
 
 #ifdef __WXOSX__
-    MoveToApplicationsFolderIfNecessary();
+    PFMoveToApplicationsFolderIfNecessary();
 
     wxSystemOptions::SetOption(wxMAC_TEXTCONTROL_USE_SPELL_CHECKER, 1);
 
@@ -454,10 +437,6 @@ bool PoeditApp::OnInit()
 
     SetDefaultCfg(wxConfig::Get());
 
-#if defined(__WXOSX__) || defined(__WXMSW__)
-    u_setDataDirectory(wxStandardPaths::Get().GetResourcesDir().mb_str());
-#endif
-
 #ifndef __WXOSX__
     wxArtProvider::Push(new PoeditArtProvider);
 #endif
@@ -471,22 +450,8 @@ bool PoeditApp::OnInit()
     s_macHelpMenuTitleName = _("&Help");
 #endif
 
-#ifdef USE_SPARKLE
-    m_nativeMacAppData->sparkleDelegate = Sparkle_Initialize();
-#endif // USE_SPARKLE
-
-#ifdef __WXMSW__
-    if (CheckForBetaUpdates())
-        win_sparkle_set_appcast_url("https://poedit.net/updates_v2/win/appcast/beta");
-    else
-        win_sparkle_set_appcast_url("https://poedit.net/updates_v2/win/appcast");
- 
-    win_sparkle_set_can_shutdown_callback(&PoeditApp::WinSparkle_CanShutdown);
-    win_sparkle_set_shutdown_request_callback(&PoeditApp::WinSparkle_Shutdown);
-    auto buildnum = GetAppBuildNumber();
-    if (!buildnum.empty())
-        win_sparkle_set_app_build_version(buildnum.wc_str());
-    win_sparkle_init();
+#ifdef HAS_UPDATES_CHECK
+    AppUpdates::Get().InitAndStart();
 #endif
 
 #ifndef __WXOSX__
@@ -552,19 +517,16 @@ int PoeditApp::OnExit()
     RecentFiles::CleanUp();
     TranslationMemory::CleanUp();
 
+#ifdef HAS_UPDATES_CHECK
+    AppUpdates::CleanUp();
+#endif
+
 #ifdef HAVE_HTTP_CLIENT
     CloudAccountClient::CleanUp();
 #endif
 
     dispatch::cleanup();
 
-#ifdef USE_SPARKLE
-    Sparkle_Cleanup();
-#endif // USE_SPARKLE
-#ifdef __WXMSW__
-    win_sparkle_cleanup();
-#endif
-
     u_cleanup();
 
     return wxApp::OnExit();
@@ -587,6 +549,19 @@ void PoeditApp::SetupLanguage()
     wxTranslations *trans = new wxTranslations();
     wxTranslations::Set(trans);
 
+    // workaround wx bug, see https://github.com/wxWidgets/wxWidgets/pull/24297
+    class PoeditTranslationsLoader : public wxFileTranslationsLoader
+    {
+    public:
+        wxArrayString GetAvailableTranslations(const wxString& domain) const override
+        {
+            auto all = wxFileTranslationsLoader::GetAvailableTranslations(domain);
+            all.push_back("en");
+            return all;
+        }
+    };
+    trans->SetLoader(new PoeditTranslationsLoader);
+
     int language = wxLANGUAGE_DEFAULT;
 
 #if NEED_CHOOSELANG_UI
@@ -625,16 +600,22 @@ void PoeditApp::SetupLanguage()
     wxString bestTrans = trans->GetBestTranslation("poedit");
     Language uiLang = Language::TryParse(bestTrans.ToStdWstring());
     UErrorCode err = U_ZERO_ERROR;
-    icu::Locale::setDefault(uiLang.ToIcu(), err);
+    uloc_setDefault(uiLang.IcuLocaleName().c_str(), &err);
 #if defined(HAVE_HTTP_CLIENT) && !defined(__WXOSX__)
     http_client::set_ui_language(uiLang.LanguageTag());
 #endif
 
+    char icuVerStr[U_MAX_VERSION_STRING_LENGTH] = {0};
+    UVersionInfo icuVer;
+    u_getVersion(icuVer);
+    u_versionToString(icuVer, icuVerStr);
+    wxLogTrace("poedit", "ICU version %s, using UI language '%s'", icuVerStr, uiLang.LanguageTag());
+
     const wxLanguageInfo *info = wxLocale::FindLanguageInfo(bestTrans);
     g_layoutDirection = info ? info->LayoutDirection : wxLayout_Default;
 
 #ifdef __WXMSW__
-    win_sparkle_set_lang(bestTrans.utf8_str());
+    AppUpdates::Get().SetLanguage(bestTrans.utf8_string());
 #endif
 
 #ifdef SUPPORTS_OTA_UPDATES
@@ -732,14 +713,13 @@ wxString PoeditApp::GetCacheDir(const wxString& category)
 
     if (localBaseDir.empty())
     {
+        localBaseDir = wxStandardPaths::Get().GetUserDir(wxStandardPaths::Dir_Cache);
     #if defined(__WXOSX__)
-        localBaseDir = wxGetHomeDir() + "/Library/Caches/net.poedit.Poedit";
-    #elif defined(__UNIX__)
-        if (!wxGetEnv("XDG_CACHE_HOME", &localBaseDir))
-            localBaseDir = wxGetHomeDir() + "/.cache";
-        localBaseDir += "/poedit";
+        localBaseDir += "/net.poedit.Poedit";
+    #elif defined(__WXMSW__)
+        localBaseDir += "\\Poedit\\Cache";
     #else
-        localBaseDir = wxStandardPaths::Get().GetUserDataDir() + wxFILE_SEP_PATH + "Cache";
+        localBaseDir += "/poedit";
     #endif
     }
 
@@ -769,7 +749,7 @@ int PoeditApp::OpenFiles(const wxArrayString& names, int lineno)
         // associated with it, so that the app can provide explanation to users
         // not familiar with the MO/PO distinction.
         auto n = name.Lower();
-        if (n.EndsWith(".mo") || n.EndsWith(".gmo"))
+        if (n.ends_with(".mo") || n.ends_with(".gmo"))
         {
             wxMessageDialog dlg(nullptr,
                                 _(L"MO files can’t be directly edited in Poedit."),
@@ -864,7 +844,7 @@ bool PoeditApp::OnCmdLineParsed(wxCmdLineParser& parser)
                 for (size_t i = 0; i < parser.GetParamCount(); i++)
                 {
                     auto fn = parser.GetParam(i);
-                    if (fn.StartsWith("poedit://"))
+                    if (fn.starts_with("poedit://"))
                         client.HandleCustomURI(fn);
                     else
                         client.OpenFile(fn, (int)lineno);
@@ -892,7 +872,7 @@ bool PoeditApp::OnCmdLineParsed(wxCmdLineParser& parser)
     for (size_t i = 0; i < parser.GetParamCount(); i++)
     {
         auto fn = parser.GetParam(i);
-        if (fn.StartsWith("poedit://"))
+        if (fn.starts_with("poedit://"))
         {
             gs_uriToHandle = fn;
         }
@@ -914,19 +894,20 @@ bool PoeditApp::OnCmdLineParsed(wxCmdLineParser& parser)
 
 void PoeditApp::HandleCustomURI(const wxString& uri)
 {
-    if (!uri.StartsWith("poedit://"))
+    if (!uri.starts_with("poedit://"))
         return;
 
 #ifdef HAVE_HTTP_CLIENT
-    if (CrowdinClient::IsOAuthCallback(uri.ToStdString()))
+    auto uri_utf8 = uri.utf8_string();
+    if (CrowdinClient::IsOAuthCallback(uri_utf8))
     {
-        CrowdinClient::Get().HandleOAuthCallback(uri.ToStdString());
+        CrowdinClient::Get().HandleOAuthCallback(uri_utf8);
         return;
     }
 
-    if (LocalazyClient::IsAuthCallback(uri.ToStdString()))
+    if (LocalazyClient::IsAuthCallback(uri_utf8))
     {
-        LocalazyClient::Get().HandleAuthCallback(uri.ToStdString())
+        LocalazyClient::Get().HandleAuthCallback(uri_utf8)
             .then_on_main([=](std::shared_ptr<LocalazyClient::ProjectInfo> projectToOpen){
                 if (projectToOpen)
                 {
@@ -987,8 +968,9 @@ BEGIN_EVENT_TABLE(PoeditApp, wxApp)
    EVT_MENU           (wxID_PREFERENCES,          PoeditApp::OnPreferences)
    EVT_MENU           (wxID_HELP,                 PoeditApp::OnHelp)
    EVT_MENU           (XRCID("menu_gettext_manual"), PoeditApp::OnGettextManual)
-#ifdef __WXMSW__
-   EVT_MENU           (XRCID("menu_check_for_updates"), PoeditApp::OnWinsparkleCheck)
+#ifdef HAS_UPDATES_CHECK
+   EVT_MENU           (XRCID("menu_check_for_updates"), PoeditApp::OnCheckForUpdates)
+   EVT_UPDATE_UI      (XRCID("menu_check_for_updates"), PoeditApp::OnEnableCheckForUpdates)
 #endif
 #ifdef __WXOSX__
    EVT_MENU           (wxID_CLOSE, PoeditApp::OnCloseWindowCommand)
@@ -1262,7 +1244,7 @@ void PoeditApp::OnAbout(wxCommandEvent&)
     about.SetVersion(wxGetApp().GetAppVersion());
     about.SetDescription(_("Poedit is an easy to use translation editor."));
 #endif
-    about.SetCopyright(L"Copyright \u00a9 1999-2023 Václav Slavík");
+    about.SetCopyright(L"Copyright \u00a9 1999-2024 Václav Slavík");
 #ifdef __WXGTK__ // other ports would show non-native about dlg
     about.SetWebSite("https://poedit.net");
     about.SetIcon(wxArtProvider::GetIcon("net.poedit.Poedit", wxART_FRAME_ICON, wxSize(128, 128)));
@@ -1401,23 +1383,14 @@ void PoeditApp::OnCloseWindowCommand(wxCommandEvent&)
 #endif // __WXOSX__
 
 
-#ifdef __WXMSW__
-
-void PoeditApp::OnWinsparkleCheck(wxCommandEvent& event)
+#ifdef HAS_UPDATES_CHECK
+void PoeditApp::OnCheckForUpdates(wxCommandEvent&)
 {
-    win_sparkle_check_update_with_ui();
+    AppUpdates::Get().CheckForUpdatesWithUI();
 }
 
-// WinSparkle callbacks:
-int PoeditApp::WinSparkle_CanShutdown()
+void PoeditApp::OnEnableCheckForUpdates(wxUpdateUIEvent& event)
 {
-    return !PoeditFrame::AnyWindowIsModified();
+    event.Enable(AppUpdates::Get().CanCheckForUpdates());
 }
-
-void PoeditApp::WinSparkle_Shutdown()
-{
-    wxGetApp().OnQuit(wxCommandEvent());
-}
-
-#endif // __WXMSW__
-
+#endif // HAS_UPDATES_CHECK
