@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2023 Vaclav Slavik
+ *  Copyright (C) 1999-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -324,16 +324,12 @@ private:
         void OnSuggestion(wxCommandEvent& event);
         void OnPreTranslateAll(wxCommandEvent& event);
 
+        void OnRemoveSameAsSourceTranslations(wxCommandEvent& event);
         void OnPurgeDeleted(wxCommandEvent& event);
 
-        void OnGoToBookmark(wxCommandEvent& event);
-        void OnSetBookmark(wxCommandEvent& event);
-
-        void AddBookmarksMenu(wxMenu *menu);
-
         void OnCompileMO(wxCommandEvent& event);
-        void OnExport(wxCommandEvent& event);
-        bool ExportCatalog(const wxString& filename);
+        void OnExportToHTML(wxCommandEvent& event);
+        bool ExportCatalogToHTML(const wxString& filename);
 
         void OnSize(wxSizeEvent& event);
 
