@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2017-2023 Vaclav Slavik
+ *  Copyright (C) 2017-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -127,9 +127,11 @@ public:
     /// Priority value for GetPriority()
     enum class Priority
     {
-        Highest = 1,
-        High    = 10,
-        Default = 100
+        Highest                     =   1,
+        CustomExtension             =   2, // customization should be highest
+        High                        =  10,
+        DefaultSpecializedExtension =  95, // for use with e.g. .blade.php extentions
+        Default                     = 100
     };
 
     /// Returns priority of the extractor
