@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2000-2023 Vaclav Slavik
+ *  Copyright (C) 2000-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,7 @@
 #include <wx/process.h>
 #include <wx/thread.h>
 #include <wx/txtstrm.h>
+#include <wx/scopedptr.h>
 #include <wx/string.h>
 #include <wx/intl.h>
 #include <wx/stdpaths.h>
@@ -40,49 +41,47 @@
 #include "concurrency.h"
 #include "gexecute.h"
 #include "errors.h"
+#include "str_helpers.h"
 
 namespace
 {
 
-#ifdef __WXOSX__
-wxString GetGettextPluginPath()
+struct CommandInvocation
 {
-    return wxStandardPaths::Get().GetPluginsDir() + "/GettextTools.bundle";
-}
-#endif // __WXOSX__
+    CommandInvocation(const wxString& command, const wxString& arguments) : exe(command), args(arguments)
+    {
+    }
 
+    CommandInvocation(const wxString& cmdline)
+    {
+        exe = cmdline.BeforeFirst(_T(' '));
+        args = cmdline.Mid(exe.length() + 1);
+    }
+
+    wxString cmdline() const
+    {
 #if defined(__WXOSX__) || defined(__WXMSW__)
-
-inline wxString GetAuxBinariesDir()
-{
-    return GetGettextPackagePath() + "/bin";
-}
-
-wxString GetPathToAuxBinary(const wxString& program)
-{
-    wxFileName path;
-    path.SetPath(GetAuxBinariesDir());
-    path.SetName(program);
-#ifdef __WXMSW__
-    path.SetExt("exe");
+        const auto fullexe = "\"" + GetGettextBinaryPath(exe) + "\"";
+#else
+        const auto& fullexe(exe);
 #endif
-    if ( path.IsFileExecutable() )
-    {
-        return wxString::Format(_T("\"%s\""), path.GetFullPath().c_str());
+        return args.empty() ? fullexe : fullexe + " " + args;
     }
-    else
-    {
-        wxLogTrace("poedit.execute",
-                   L"%s doesn’t exist, falling back to %s",
-                   path.GetFullPath().c_str(),
-                   program.c_str());
-        return program;
-    }
-}
-#endif // __WXOSX__ || __WXMSW__
+
+    wxString exe;
+    wxString args;
+};
 
 
-bool ReadOutput(wxInputStream& s, wxArrayString& out)
+struct ProcessOutput
+{
+    long return_code = -1;
+    std::vector<wxString> std_out;
+    std::vector<wxString> std_err;
+};
+
+
+bool ReadOutput(wxInputStream& s, std::vector<wxString>& out)
 {
     // the stream could be already at EOF or in wxSTREAM_BROKEN_PIPE state
     s.Reset();
@@ -114,47 +113,98 @@ bool ReadOutput(wxInputStream& s, wxArrayString& out)
     return true;
 }
 
-std::pair<long, wxArrayString> DoExecuteGettextImpl(const wxString& cmdline_)
+
+ProcessOutput ExecuteCommandAndCaptureOutput(const CommandInvocation& cmd, const wxExecuteEnv *env)
 {
-    wxArrayString gstderr;
-    wxExecuteEnv env;
-    wxString cmdline(cmdline_);
-
-#if defined(__WXOSX__) || defined(__WXMSW__)
-    wxString binary = cmdline.BeforeFirst(_T(' '));
-    cmdline = GetPathToAuxBinary(binary) + cmdline.Mid(binary.length());
-    wxGetEnvMap(&env.env);
-    env.env["OUTPUT_CHARSET"] = "UTF-8";
-
-    wxString lang = wxTranslations::Get()->GetBestTranslation("gettext-tools");
-	if ( !lang.empty() )
-        env.env["LANG"] = lang;
-#endif // __WXOSX__ || __WXMSW__
+    ProcessOutput pout;
+    auto cmdline = cmd.cmdline();
 
     wxLogTrace("poedit.execute", "executing: %s", cmdline.c_str());
 
     wxScopedPtr<wxProcess> process(new wxProcess);
     process->Redirect();
 
-    long retcode = wxExecute(cmdline, wxEXEC_BLOCK | wxEXEC_NODISABLE | wxEXEC_NOEVENTS, process.get(), &env);
-    if (retcode != 0)
+    pout.return_code = wxExecute(cmdline, wxEXEC_BLOCK | wxEXEC_NODISABLE | wxEXEC_NOEVENTS, process.get(), env);
+    if (pout.return_code != 0)
     {
-        wxLogTrace("poedit.execute", "  execution of command failed with exit code %d: %s", (int)retcode, cmdline.c_str());
+        wxLogTrace("poedit.execute", "  execution of command failed with exit code %d: %s", (int)pout.return_code, cmdline.c_str());
     }
 
-	wxInputStream *std_err = process->GetErrorStream();
-    if ( std_err && !ReadOutput(*std_err, gstderr) )
-        retcode = -1;
+    wxInputStream *std_out = process->GetInputStream();
+    if ( std_out && !ReadOutput(*std_out, pout.std_out) )
+        pout.return_code = -1;
 
-    if ( retcode == -1 )
+    wxInputStream *std_err = process->GetErrorStream();
+    if ( std_err && !ReadOutput(*std_err, pout.std_err) )
+        pout.return_code = -1;
+
+    if ( pout.return_code == -1 )
     {
         BOOST_THROW_EXCEPTION(Exception(wxString::Format(_("Cannot execute program: %s"), cmdline.c_str())));
     }
 
-    return std::make_pair(retcode, gstderr);
+    return pout;
 }
 
-std::pair<long, wxArrayString> DoExecuteGettext(const wxString& cmdline)
+
+#define GETTEXT_VERSION_NUM(x, y, z)  ((x*1000*1000) + (y*1000) + (z))
+
+// Determine gettext version, return it in the form of XXXYYYZZZ number for version x.y.z
+uint32_t GetGettextVersion()
+{
+    static uint32_t s_version = 0;
+    if (!s_version)
+    {
+        // set old enough fallback version
+        s_version = GETTEXT_VERSION_NUM(0, 18, 0);
+
+        // then determine actual version
+        CommandInvocation msgfmt("msgfmt", "--version");
+        auto p = ExecuteCommandAndCaptureOutput(msgfmt, nullptr);
+        if (p.return_code == 0 && p.std_out.size() > 1)
+        {
+            auto line = str::to_utf8(p.std_out[0]);
+            std::smatch m;
+            if (std::regex_match(line, m, std::regex(R"(.* (([0-9]+)\.([0-9]+)(\.([0-9]+))?)$)")))
+            {
+                const int x = std::stoi(m.str(2));
+                const int y = std::stoi(m.str(3));
+                const int z = m[5].matched ? std::stoi(m.str(5)) : 0;
+                s_version = GETTEXT_VERSION_NUM(x, y, z);
+            }
+        }
+    }
+    return s_version;
+}
+
+
+ProcessOutput DoExecuteGettextImpl(CommandInvocation cmd)
+{
+    // gettext 0.22 started converting MO files to UTF-8 by default. Don't do that.
+    // See https://git.savannah.gnu.org/gitweb/?p=gettext.git;a=commit;h=5412a4f79929004cb6db15d545e07dc953330e8d
+    if (cmd.exe == "msgfmt" && GetGettextVersion() >= GETTEXT_VERSION_NUM(0, 22, 0))
+    {
+        cmd.args = "--no-convert " + cmd.args;
+    }
+
+#if defined(__WXOSX__) || defined(__WXMSW__)
+    wxExecuteEnv env;
+    wxGetEnvMap(&env.env);
+    env.env["OUTPUT_CHARSET"] = "UTF-8";
+
+    wxString lang = wxTranslations::Get()->GetBestTranslation("gettext-tools");
+    if ( lang.starts_with("en@") )
+        lang = "en"; // don't want things like en@blockquot
+	if ( !lang.empty() )
+        env.env["LANG"] = lang;
+    return ExecuteCommandAndCaptureOutput(cmd, &env);
+#else // Unix
+    return ExecuteCommandAndCaptureOutput(cmd, nullptr);
+#endif
+}
+
+
+ProcessOutput DoExecuteGettext(const wxString& cmdline)
 {
 #if wxUSE_GUI
     if (wxThread::IsMain())
@@ -179,7 +229,7 @@ void LogUnrecognizedError(const wxString& err)
     //   Warning: Failed to set locale category LC_NUMERIC to de.
     //   Warning: Failed to set locale category LC_TIME to de.
     //   ...etc...
-    if (err.StartsWith("Warning: Failed to set locale category"))
+    if (err.starts_with("Warning: Failed to set locale category"))
         return;
 #endif // __WXOSX__
 
@@ -191,12 +241,10 @@ void LogUnrecognizedError(const wxString& err)
 
 bool ExecuteGettext(const wxString& cmdline)
 {
-    wxArrayString gstderr;
-    long retcode;
-    std::tie(retcode, gstderr) = DoExecuteGettext(cmdline);
+    auto output = DoExecuteGettext(cmdline);
 
     wxString pending;
-    for (auto& ln: gstderr)
+    for (auto& ln: output.std_err)
     {
         if (ln.empty())
             continue;
@@ -218,19 +266,17 @@ bool ExecuteGettext(const wxString& cmdline)
     if (!pending.empty())
         LogUnrecognizedError(pending);
 
-    return retcode == 0;
+    return output.return_code == 0;
 }
 
 
 bool ExecuteGettextAndParseOutput(const wxString& cmdline, GettextErrors& errors)
 {
-    wxArrayString gstderr;
-    long retcode;
-    std::tie(retcode, gstderr) = DoExecuteGettext(cmdline);
+    auto output = DoExecuteGettext(cmdline);
 
     static const std::wregex RE_ERROR(L".*\\.po:([0-9]+)(:[0-9]+)?: (.*)");
 
-    for (const auto& ewx: gstderr)
+    for (const auto& ewx: output.std_err)
     {
         const auto e = ewx.ToStdWstring();
         wxLogTrace("poedit", "  stderr: %s", e.c_str());
@@ -256,7 +302,7 @@ bool ExecuteGettextAndParseOutput(const wxString& cmdline, GettextErrors& errors
         }
     }
 
-    return retcode == 0;
+    return output.return_code == 0;
 }
 
 
@@ -271,13 +317,37 @@ wxString QuoteCmdlineArg(const wxString& s)
 
 
 #if defined(__WXOSX__) || defined(__WXMSW__)
+
 wxString GetGettextPackagePath()
 {
 #if defined(__WXOSX__)
-    return GetGettextPluginPath() + "/Contents/MacOS";
+    auto plugin = wxStandardPaths::Get().GetPluginsDir() + "/GettextTools.bundle";
+    return plugin + "/Contents/MacOS";
 #elif defined(__WXMSW__)
     return wxStandardPaths::Get().GetDataDir() + wxFILE_SEP_PATH + "GettextTools";
 #endif
 }
-#endif // __WXOSX__ || __WXMSW__
 
+wxString GetGettextBinaryPath(const wxString& program)
+{
+    wxFileName path;
+    path.SetPath(GetGettextPackagePath() + "/bin");
+    path.SetName(program);
+#ifdef __WXMSW__
+    path.SetExt("exe");
+#endif
+    if ( path.IsFileExecutable() )
+    {
+        return path.GetFullPath();
+    }
+    else
+    {
+        wxLogTrace("poedit.execute",
+                   L"%s doesn’t exist, falling back to %s",
+                   path.GetFullPath().c_str(),
+                   program.c_str());
+        return program;
+    }
+}
+
+#endif // __WXOSX__ || __WXMSW__
