@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2023 Vaclav Slavik
+ *  Copyright (C) 2013-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,9 @@
 
 #include "language.h"
 
+#include "str_helpers.h"
+#include "unicode_helpers.h"
+
 #include <cctype>
 #include <algorithm>
 #include <unordered_map>
@@ -35,13 +38,10 @@
 
 #include <boost/algorithm/string.hpp>
 
-#include <unicode/locid.h>
-#include <unicode/coll.h>
 #include <unicode/utypes.h>
 
 #include <wx/filename.h>
 
-#include "str_helpers.h"
 #include "pluralforms/pl_evaluate.h"
 
 #ifdef HAVE_CLD2
@@ -95,7 +95,7 @@ void TryNormalize(std::wstring& s)
 bool IsISOLanguage(const std::string& s)
 {
     const char *test = s.c_str();
-    for (const char * const* i = icu::Locale::getISOLanguages(); *i != nullptr; ++i)
+    for (const char * const* i = uloc_getISOLanguages(); *i != nullptr; ++i)
     {
         if (strcmp(test, *i) == 0)
             return true;
@@ -106,7 +106,7 @@ bool IsISOLanguage(const std::string& s)
 bool IsISOCountry(const std::string& s)
 {
     const char *test = s.c_str();
-    for (const char * const* i = icu::Locale::getISOCountries(); *i != nullptr; ++i)
+    for (const char * const* i = uloc_getISOCountries(); *i != nullptr; ++i)
     {
         if (strcmp(test, *i) == 0)
             return true;
@@ -114,10 +114,27 @@ bool IsISOCountry(const std::string& s)
     return false;
 }
 
+// Get locale display name or at least language
+template<typename T>
+auto GetDisplayNameOrLanguage(const char *locale, const char *displayLocale)
+{
+    UErrorCode err = U_ZERO_ERROR;
+    UChar buf[512] = {0};
+    uloc_getDisplayName(locale, displayLocale, buf, std::size(buf), &err);
+    if (U_FAILURE(err) || str::empty(buf))
+    {
+        err = U_ZERO_ERROR;
+        uloc_getDisplayLanguage(locale, displayLocale, buf, std::size(buf), &err);
+    }
+
+    return str::to<T>(buf);
+}
+
+
 // Mapping of names to their respective ISO codes.
 struct DisplayNamesData
 {
-    typedef std::unordered_map<std::wstring, std::string> Map;
+    typedef std::unordered_map<std::u16string, std::string> Map;
     Map names, namesEng;
     std::vector<std::wstring> sortedNames;
 };
@@ -129,36 +146,44 @@ const DisplayNamesData& GetDisplayNamesData()
     static DisplayNamesData data;
 
     std::call_once(of_namesList, [=]{
-        auto locEng = icu::Locale::getEnglish();
-        std::vector<icu::UnicodeString> names;
         std::set<std::string> foundCodes;
 
-        int32_t count;
-        const icu::Locale *loc = icu::Locale::getAvailableLocales(count);
-        names.reserve(count);
+        int32_t count = uloc_countAvailable();
+        data.sortedNames.reserve(count);
 
-        for (int i = 0; i < count; i++, loc++)
+        char language[128] = {0};
+        char script[128] = {0};
+        char country[128] = {0};
+        char variant[128] = {0};
+
+        for (int i = 0; i < count; i++)
         {
-            auto language = loc->getLanguage();
-            auto script = loc->getScript();
-            auto country = loc->getCountry();
-            auto variant = loc->getVariant();
+            const char *locale = uloc_getAvailable(i);
+
+            UErrorCode err = U_ZERO_ERROR;
+            uloc_getLanguage(locale, language, std::size(language), &err);
+            uloc_getScript(locale, script, std::size(script), &err);
+            uloc_getCountry(locale, country, std::size(country), &err);
+            uloc_getVariant(locale, variant, std::size(variant), &err);
 
             // TODO: for now, ignore variants here and in FormatForRoundtrip(),
             //       because translating them between gettext and ICU is nontrivial
-            if (variant != nullptr && *variant != '\0')
+            if (!str::empty(variant))
                 continue;
 
-            icu::UnicodeString s;
-            loc->getDisplayName(s);
-            names.push_back(s);
+            UChar buf[512] = {0};
+            err = U_ZERO_ERROR;
+            uloc_getDisplayName(locale, nullptr, buf, std::size(buf), &err);
+
+            data.sortedNames.emplace_back(str::to_wstring(buf));
+            auto foldedName = unicode::fold_case_to_type<std::u16string>(buf);
 
             if (strcmp(language, "zh") == 0 && *country == '\0')
             {
                 if (strcmp(script, "Hans") == 0)
-                    country = "CN";
+                    strncpy(country, "CN", std::size(country));
                 else if (strcmp(script, "Hant") == 0)
-                    country = "TW";
+                    strncpy(country, "TW", std::size(country));
             }
 
             std::string code(language);
@@ -182,65 +207,42 @@ const DisplayNamesData& GetDisplayNamesData()
             }
 
             foundCodes.insert(code);
+            data.names[foldedName] = code;
+ 
+            err = U_ZERO_ERROR;
+            uloc_getDisplayName(locale, ULOC_ENGLISH, buf, std::size(buf), &err);
+            auto foldedEngName = unicode::fold_case_to_type<std::u16string>(buf);
 
-            s.foldCase();
-            data.names[str::to_wstring(s)] = code;
-
-            loc->getDisplayName(locEng, s);
-            s.foldCase();
-            data.namesEng[str::to_wstring(s)] = code;
+            data.namesEng[foldedEngName] = code;
         }
 
         // add languages that are not listed as locales in ICU:
-        for (const char * const* i = icu::Locale::getISOLanguages(); *i != nullptr; ++i)
+        for (const char * const* i = uloc_getISOLanguages(); *i != nullptr; ++i)
         {
             const char *code = *i;
             if (foundCodes.find(code) != foundCodes.end())
                 continue;
 
-            icu::Locale langLoc(code);
-            if (strcmp(code, langLoc.getLanguage()) != 0)
+            UErrorCode err = U_ZERO_ERROR;
+            uloc_getLanguage(code, language, std::size(language), &err);
+            if (U_FAILURE(err) || strcmp(code, language) != 0)
                 continue; // e.g. 'und' for undetermined language
 
-            icu::UnicodeString name;
-            if (langLoc.getDisplayName(name).isEmpty())
-                langLoc.getDisplayLanguage(name);
-            if (name.isEmpty())
+            auto isoName = GetDisplayNameOrLanguage<std::wstring>(code, nullptr);
+            if (isoName.empty())
                 continue;
 
-            names.push_back(name);
+            data.sortedNames.push_back(isoName);
+            data.names[unicode::fold_case_to_type<std::u16string>(isoName)] = code;
 
-            name.foldCase();
-            data.names[str::to_wstring(name)] = code;
-
-            if (langLoc.getDisplayName(locEng, name).isEmpty())
-                langLoc.getDisplayLanguage(locEng, name);
-            name.foldCase();
-            data.namesEng[str::to_wstring(name)] = code;
+            auto isoEngName = GetDisplayNameOrLanguage<std::wstring>(code, ULOC_ENGLISH);
+            if (!isoEngName.empty())
+                data.namesEng[unicode::fold_case_to_type<std::u16string>(isoEngName)] = code;
         }
 
         // sort the names alphabetically for data.sortedNames:
-        UErrorCode err = U_ZERO_ERROR;
-        std::unique_ptr<icu::Collator> coll(icu::Collator::createInstance(err));
-        if (coll)
-        {
-            coll->setStrength(icu::Collator::SECONDARY); // case insensitive
-
-            std::sort(names.begin(), names.end(),
-                      [&coll](const icu::UnicodeString& a, const icu::UnicodeString& b){
-                          UErrorCode e = U_ZERO_ERROR;
-                          return coll->compare(a, b, e) == UCOL_LESS;
-                      });
-        }
-        else
-        {
-            std::sort(names.begin(), names.end());
-        }
-
-        // convert into std::wstring
-        data.sortedNames.reserve(names.size());
-        for (auto s: names)
-            data.sortedNames.push_back(str::to_wstring(s));
+        unicode::Collator coll(unicode::Collator::case_insensitive);
+        std::sort(data.sortedNames.begin(), data.sortedNames.end(), std::ref(coll));
     });
 
     return data;
@@ -384,9 +386,9 @@ Language Language::TryParse(const std::wstring& s)
     if (IsValidCode(s))
         return Language(s);
 
-    if (s == "zh-Hans")
+    if (s == L"zh-Hans")
         return Language("zh_CN");
-    else if (s == "zh-Hant")
+    else if (s == L"zh-Hant")
         return Language("zh_TW");
 
     // Is it a standard language code?
@@ -400,9 +402,7 @@ Language Language::TryParse(const std::wstring& s)
 
     // If not, perhaps it's a human-readable name (perhaps coming from the language control)?
     auto names = GetDisplayNamesData();
-    icu::UnicodeString s_icu = str::to_icu(s);
-    s_icu.foldCase();
-    std::wstring folded = str::to_wstring(s_icu);
+    auto folded = unicode::fold_case_to_type<std::u16string>(s);
     auto i = names.names.find(folded);
     if (i != names.names.end())
         return Language(i->second);
@@ -445,8 +445,8 @@ Language Language::FromLanguageTag(const std::string& tag)
 
     char locale[512];
     UErrorCode status = U_ZERO_ERROR;
-    uloc_forLanguageTag(tag.c_str(), locale, 512, NULL, &status);
-    if (U_FAILURE(status))
+    auto len = uloc_forLanguageTag(tag.c_str(), locale, 512, NULL, &status);
+    if (U_FAILURE(status) || !len)
         return Language();
 
     Language lang;
@@ -528,40 +528,27 @@ int Language::nplurals() const
 }
 
 
-icu::Locale Language::ToIcu() const
-{
-    if (!IsValid())
-        return icu::Locale::getEnglish();
-
-    return icu::Locale(IcuLocaleName().c_str());
-}
-
-
 wxString Language::DisplayName() const
 {
-    auto loc = ToIcu();
-    icu::UnicodeString s;
-    if (loc.getDisplayName(s).isEmpty())
-        loc.getDisplayLanguage(s);
-    return str::to_wx(s);
+    return GetDisplayNameOrLanguage<wxString>(m_icuLocale.c_str(), nullptr);
 }
 
 wxString Language::LanguageDisplayName() const
 {
-    icu::UnicodeString s;
-    ToIcu().getDisplayLanguage(s);
-    return str::to_wx(s);
+    UErrorCode err = U_ZERO_ERROR;
+    UChar buf[512] = {0};
+    uloc_getDisplayLanguage(m_icuLocale.c_str(), nullptr, buf, std::size(buf), &err);
+    return str::to_wx(buf);
 }
 
 wxString Language::DisplayNameInItself() const
 {
-    auto loc = ToIcu();
-    icu::UnicodeString s;
-    if (loc.getDisplayName(loc, s).isEmpty())
-        loc.getDisplayLanguage(loc, s);
-    if (s.isEmpty())
-        return DisplayName(); // fall back to current locale's name, better than nothing
-    return str::to_wx(s);
+    auto name = GetDisplayNameOrLanguage<wxString>(m_icuLocale.c_str(), m_icuLocale.c_str());
+    if (!name.empty())
+        return name;
+
+    // fall back to current locale's name, better than nothing
+    return DisplayName();
 }
 
 wxString Language::FormatForRoundtrip() const
