@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2023 Vaclav Slavik
+ *  Copyright (C) 2013-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,6 @@
 #include <memory>
 #include <string>
 #include <vector>
-#include <unicode/locid.h>
 
 class PluralFormsCalculator;
 class PluralFormsExpr;
@@ -71,8 +70,6 @@ public:
 
     /// Returns name of the locale suitable for ICU
     std::string IcuLocaleName() const { return m_icuLocale; }
-    /// Returns ICU equivalent of the language info
-    icu::Locale ToIcu() const;
 
     /// Returns name of this language suitable for display to the user in current UI language
     wxString DisplayName() const;
@@ -173,8 +170,7 @@ public:
     static Language TryDetectFromText(const char *buffer, size_t len,
                                       Language probableLanguage = Language());
 
-    template<typename T>
-    static Language TryDetectFromText(const T& str, Language probableLanguage = Language())
+    static Language TryDetectFromText(const std::string& str, Language probableLanguage = Language())
     {
         return TryDetectFromText(str.data(), str.length(), probableLanguage);
     }
