@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2023 Vaclav Slavik
+ *  Copyright (C) 1999-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -25,10 +25,7 @@
 
 #include "menus.h"
 
-#ifdef __WXOSX__
-#include "macos_helpers.h"
-#endif
-
+#include "app_updates.h"
 #include "recent_files.h"
 #include "str_helpers.h"
 
@@ -138,22 +135,21 @@ void MenusManager::TweakOSXMenuBar(wxMenuBar *bar)
     wxMenu *apple = bar->OSXGetAppleMenu();
     if (apple)
     {
-        apple->Insert(3, XRCID("menu_manager"), _("Catalogs Manager"));
-        apple->InsertSeparator(3);
-    }
+        apple->Insert(1, XRCID("menu_check_for_updates"), _(L"Check for Updates…"));
 
-#if USE_SPARKLE
-    Sparkle_AddMenuItem(apple->GetHMenu(), _(L"Check for Updates…").utf8_str());
-#endif
+        apple->Insert(4, XRCID("menu_manager"), _("Catalogs Manager"));
+        apple->InsertSeparator(4);
+    }
 
     if (@available(macOS 13.0, *))
+    {
+        // Correct label is already used
+    }
+    else
     {
         wxMenuItem *prefsItem = bar->FindItem(wxID_PREFERENCES);
         if (prefsItem)
-        {
-            // TRANSLATORS: This is Settings app menu that replaces Preferences in macOS 13 Ventura or newer, and should be translated EXACTLY as in macOS. If you don't use macOS and can't check, please leave it untranslated.
-            prefsItem->SetItemLabel(_(L"Settings…") + "\tCtrl+,");
-        }
+            prefsItem->SetItemLabel(_(L"&Preferences…") + "\tCtrl+,");
     }
 
     wxMenu *fileMenu = nullptr;
