@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2000-2023 Vaclav Slavik
+ *  Copyright (C) 2000-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -52,10 +52,7 @@
 #include <wx/xrc/xmlres.h>
 #include <wx/numformatter.h>
 
-#if !wxCHECK_VERSION(3,1,0)
-    #define CenterVertical() Center()
-#endif
-
+#include "app_updates.h"
 #include "edapp.h"
 #include "edframe.h"
 #include "catalog.h"
@@ -74,17 +71,6 @@
 #include "customcontrols.h"
 #include "unicode_helpers.h"
 
-#ifdef __WXMSW__
-#include <winsparkle.h>
-#endif
-
-#ifdef USE_SPARKLE
-#include "macos_helpers.h"
-#endif // USE_SPARKLE
-
-#if defined(USE_SPARKLE) || defined(__WXMSW__)
-    #define HAS_UPDATES_CHECK
-#endif
 
 // Handling of different page icons
 #ifdef __WXOSX__
@@ -418,7 +404,7 @@ public:
         mergeSizer->AddSpacer(PX(5));
         mergeSizer->Add(m_mergeBehavior, wxSizerFlags().Center()
                     #ifdef __WXOSX__ // BORDER_WIN would reset this padding otherwise
-                        .Border(wxTOP, AboveCheckboxPadding())
+                        .Border(wxTOP, AboveChoicePadding())
                     #else
                         .BORDER_WIN(wxBOTTOM, 1)
                     #endif
@@ -431,7 +417,7 @@ public:
 
         auto learnMore = new LearnMoreLink(this, "https://poedit.net/trac/wiki/Doc/TranslationMemory");
         sizer->AddSpacer(PX(3));
-        sizer->Add(learnMore, wxSizerFlags().Border(wxLEFT, PX(ExplanationLabel::CHECKBOX_INDENT + LearnMoreLink::EXTRA_INDENT)));
+        sizer->Add(learnMore, wxSizerFlags().Border(wxLEFT, PX(ExplanationLabel::CHECKBOX_INDENT)));
         sizer->AddSpacer(PX(10));
 
 #ifdef __WXOSX__
@@ -507,10 +493,10 @@ private:
 
     void OnManageTM(wxCommandEvent& e)
     {
-        static const auto idLearn = wxNewId();
-        static const auto idImportTMX = wxNewId();
-        static const auto idExportTMX = wxNewId();
-        static const auto idReset = wxNewId();
+        static wxWindowIDRef idLearn = NewControlId();
+        static wxWindowIDRef idImportTMX = NewControlId();
+        static wxWindowIDRef idExportTMX = NewControlId();
+        static wxWindowIDRef idReset = NewControlId();
 
         wxMenu menu;
 #ifdef __WXOSX__
@@ -1122,29 +1108,16 @@ public:
 
     void InitValues(const wxConfigBase&) override
     {
-#ifdef USE_SPARKLE
-        m_updates->SetValue((bool)UserDefaults_GetBoolValue("SUEnableAutomaticChecks"));
-#endif
-#ifdef __WXMSW__
-        m_updates->SetValue(win_sparkle_get_automatic_check_for_updates() != 0);
-#endif
-        m_beta->SetValue(wxGetApp().CheckForBetaUpdates());
+        m_updates->SetValue(AppUpdates::Get().AutomaticChecksEnabled());
+        m_beta->SetValue(Config::CheckForBetaUpdates());
     }
 
-    void SaveValues(wxConfigBase& cfg) override
+    void SaveValues(wxConfigBase&) override
     {
-        cfg.Write("check_for_beta_updates", m_beta->GetValue());
-#ifdef __WXMSW__
-        win_sparkle_set_automatic_check_for_updates(m_updates->GetValue());
-        // FIXME: don't duplicate this code from PoeditApp::OnInit()
-        if (m_beta->GetValue())
-            win_sparkle_set_appcast_url("https://poedit.net/updates_v2/win/appcast/beta");
-        else
-            win_sparkle_set_appcast_url("https://poedit.net/updates_v2/win/appcast");
-#endif
-#ifdef USE_SPARKLE
-        UserDefaults_SetBoolValue("SUEnableAutomaticChecks", m_updates->GetValue());
-#endif
+        // NB: Must be done first, before calling AppUpdates methods!
+        Config::CheckForBetaUpdates(m_beta->GetValue());
+
+        AppUpdates::Get().EnableAutomaticChecks(m_beta->GetValue());
     }
 
 private:
@@ -1177,10 +1150,11 @@ public:
         auto crlfbox = new wxBoxSizer(wxHORIZONTAL);
         sizer->Add(crlfbox, wxSizerFlags().Expand().PXBorder(wxTOP));
         crlfbox->Add(new wxStaticText(this, wxID_ANY, _("Line endings:")), wxSizerFlags().Center().BORDER_WIN(wxTOP, PX(1)));
+        crlfbox->AddSpacer(PX(5));
         m_crlf = new wxChoice(this, wxID_ANY);
         m_crlf->Append(_("Unix (recommended)"));
         m_crlf->Append(_("Windows"));
-        crlfbox->Add(m_crlf, wxSizerFlags(1).Center().BORDER_MACOS(wxLEFT, PX(3)).BORDER_WIN(wxLEFT, PX(5)));
+        crlfbox->Add(m_crlf, wxSizerFlags(1).Center().Border(wxTOP, AboveChoicePadding()));
 
         /// TRANSLATORS: Followed by text control for entering number; wraps text at given width
         m_wrap = new wxCheckBox(this, wxID_ANY, _("Wrap at:"));
@@ -1272,3 +1246,9 @@ std::unique_ptr<PoeditPreferencesEditor> PoeditPreferencesEditor::Create()
     return p;
 }
 
+PoeditPreferencesEditor::PoeditPreferencesEditor()
+#if defined(__WXMSW__) || defined(__WXOSX__)
+    : wxPreferencesEditor(_("Settings"))
+#endif
+{
+}
