@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2017-2023 Vaclav Slavik
+ *  Copyright (C) 2017-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -367,7 +367,7 @@ public:
         }
         else if (!s_punct && t_punct)
         {
-            if (t_last == '.' && (source.EndsWith("st") || source.EndsWith("nd") || source.EndsWith("th")))
+            if (t_last == '.' && (source.ends_with("st") || source.ends_with("nd") || source.ends_with("th")))
             {
                 // special-case English ordinals to languages that use [number].
             }
@@ -380,7 +380,7 @@ public:
         }
         else if (s_punct && t_punct && s_last != t_last)
         {
-            if (IsEquivalent(L'…', t_last) && source.EndsWith("..."))
+            if (IsEquivalent(L'…', t_last) && source.ends_with("..."))
             {
                 // as a special case, allow translating ... (3 dots) as … (ellipsis)
             }
@@ -479,6 +479,26 @@ private:
             if (src == '.')
                 return trans == L'։' || trans == L':';
         }
+        else if (m_lang == "ti" || m_lang == "am")
+        {
+            // Handle Geʽez script script (used by Amharic, Tigrinya and more languages) punctuation:
+            switch (trans)
+            {
+                case L'።':
+                    return src == '.';
+                case L'፣':
+                    return src == ',';
+                case L'፤':
+                    return src == ';';
+                case L'፥':
+                    return src == ':';
+                case L'፧':
+                    return src == '?';
+                default:
+                    break;
+
+            }
+        }
 
         return false;
     }
@@ -559,9 +579,6 @@ std::vector<std::pair<std::string, wxString>> QAChecker::GetMetadata()
 
 int QAChecker::Check(Catalog& catalog)
 {
-    // TODO: parallelize this (make async tasks for chunks of the catalog)
-    //       doing it per-checker is MT-unsafe with API that calls SetIssue()!
-
     int issues = 0;
 
     for (auto& i: catalog.items())
