@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2023 Vaclav Slavik
+ *  Copyright (C) 2014-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -33,6 +33,7 @@
 #include "configuration.h"
 #include "errors.h"
 #include "hidpi.h"
+#include "static_ids.h"
 #include "utility.h"
 #include "unicode_helpers.h"
 
@@ -50,10 +51,6 @@
 #include <wx/time.h>
 #include <wx/wupdlock.h>
 
-#if !wxCHECK_VERSION(3,1,0)
-    #define CenterVertical() Center()
-#endif
-
 #include <algorithm>
 
 
@@ -140,7 +137,7 @@ public:
         m_innerSizer->Add(new ExplanationLabel(parent, _("The old source text (before it changed during an update) that the now-inaccurate translation corresponds to.")),
                      wxSizerFlags().Expand());
         m_innerSizer->AddSpacer(PX(5));
-        m_text = new SelectableAutoWrappingText(parent, "");
+        m_text = new SelectableAutoWrappingText(parent, WinID::PreviousSourceText, "");
         m_innerSizer->Add(m_text, wxSizerFlags().Expand());
     }
 
@@ -166,7 +163,7 @@ public:
         : SidebarBlock(parent, _("Notes for translators"))
     {
         m_innerSizer->AddSpacer(PX(5));
-        m_comment = new SelectableAutoWrappingText(parent, "");
+        m_comment = new SelectableAutoWrappingText(parent, WinID::NotesForTranslator, "");
         m_innerSizer->Add(m_comment, wxSizerFlags().Expand());
     }
 
@@ -178,7 +175,7 @@ public:
     void Update(const CatalogItemPtr& item) override
     {
         auto comment = wxJoin(item->GetExtractedComments(), '\n', '\0');
-        if (comment.StartsWith("TRANSLATORS:") || comment.StartsWith("translators:"))
+        if (comment.starts_with("TRANSLATORS:") || comment.starts_with("translators:"))
         {
             comment.Remove(0, 12);
             if (!comment.empty() && comment[0] == ' ')
@@ -199,7 +196,7 @@ public:
         : SidebarBlock(parent, _("Comment"))
     {
         m_innerSizer->AddSpacer(PX(5));
-        m_comment = new SelectableAutoWrappingText(parent, "");
+        m_comment = new SelectableAutoWrappingText(parent, WinID::TranslatorComment, "");
         m_innerSizer->Add(m_comment, wxSizerFlags().Expand());
     }
 
@@ -270,7 +267,7 @@ public:
         m_parentBlock = block;
         m_isHighlighted = false;
         m_icon = new StaticBitmap(this, "SuggestionTMTemplate");
-        m_text = new AutoWrappingText(this, "TEXT");
+        m_text = new AutoWrappingText(this, wxID_ANY, "TEXT");
         m_info = new InfoStaticText(this);
         m_moreActions = new ImageButton(this, "DownvoteTemplate");
 
@@ -442,10 +439,8 @@ private:
             {
 #if defined(__WXOSX__)
                 gc->DrawRoundedRectangle(rect.x, rect.y, rect.width, rect.height, PX(5));
-#elif wxCHECK_VERSION(3,1,1)
-                gc->DrawRoundedRectangle(rect.x, rect.y, rect.width, rect.height, PX(2));
 #else
-                gc->DrawRectangle(rect.x, rect.y, rect.width, rect.height);
+                gc->DrawRoundedRectangle(rect.x, rect.y, rect.width, rect.height, PX(2));
 #endif
             }
         }
@@ -480,7 +475,7 @@ private:
 
         auto sidebar = m_sidebar;
         auto suggestion = m_value;
-        static const auto idDelete = wxNewId();
+        static wxWindowIDRef idDelete = NewControlId();
 
         wxMenu menu;
 #ifdef __WXOSX__
