@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2023 Vaclav Slavik
+ *  Copyright (C) 2013-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -55,6 +55,8 @@ const wchar_t* Symbol_ChromClose = L"\U0000E8BB";
 wxBitmap RenderButton(const wxSize& size, const wxFont& font, const wxColour& bg, const wxColour& fg)
 {
     wxBitmap bmp(size);
+    bmp.SetScaleFactor(HiDPIScalingFactor());
+
     {
         wxMemoryDC dc(bmp);
         dc.SetBackground(bg);
@@ -195,9 +197,6 @@ bool TitlelessWindowBase<T>::SetBackgroundColour(const wxColour& clr)
 template<typename T>
 bool TitlelessWindowBase<T>::ShouldRemoveChrome()
 {
-    if (!IsWindows10OrGreater())
-        return false;
-
     if (!wxUxThemeIsActive())
         return false;
 
