@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2010-2023 Vaclav Slavik
+ *  Copyright (C) 2010-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -113,7 +113,7 @@ wxFileName MakeFileName(const wxString& path)
         fn.AssignDir(path);
     else
         fn.Assign(path);
-    fn.Normalize(wxPATH_NORM_DOTS | wxPATH_NORM_ABSOLUTE);
+    fn.MakeAbsolute();
     return fn;
 }
 
@@ -460,14 +460,7 @@ void RestoreWindowState(wxTopLevelWindow *win, const wxSize& defaultSize, int fl
 #endif // __WXMSW__/__WXOSX__
 
     // If the window is larger than current screen, resize it to fit:
-#if wxCHECK_VERSION(3,1,0)
     wxDisplay display(win);
-#else
-    int display_num = wxDisplay::GetFromWindow(win);
-    if (display_num == wxNOT_FOUND)
-        return;
-    wxDisplay display(display_num);
-#endif
     if (!display.IsOk())
         return;
 
