@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2010-2023 Vaclav Slavik
+ *  Copyright (C) 2010-2024 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -68,32 +68,18 @@
 #endif
 
 #ifdef __WXOSX__
-    inline int AboveCheckboxPadding()
+    inline int AboveChoicePadding()
     {
         if (__builtin_available(macOS 11.0, *))
-            return 4;
+            return 2;
         else
             return 0;
     }
 #else
-    #define AboveCheckboxPadding()  0
+    #define AboveChoicePadding()  0
 #endif
 
 
-// ----------------------------------------------------------------------
-// Platform version checks
-// ----------------------------------------------------------------------
-
-#ifdef __WXMSW__
-
-inline bool IsWindows10OrGreater()
-{
-    int osmajor = 0;
-    return wxGetOsVersion(&osmajor, nullptr) == wxOS_WINDOWS_NT && osmajor >= 10;
-}
-
-#endif // __WXMSW__
-
 // ----------------------------------------------------------------------
 // Misc helpers
 // ----------------------------------------------------------------------
@@ -187,7 +173,7 @@ wxFileName MakeFileName(const wxString& path);
 
 inline wxFileName MakeFileName(wxFileName fn)
 {
-    fn.Normalize(wxPATH_NORM_DOTS | wxPATH_NORM_ABSOLUTE);
+    fn.MakeAbsolute();
     return fn;
 }
 
